//------------------------ CSPEvent.h: -----------------------

//*************************************************************
//*****       Copyright 2002 - 2008 Micronovae Ltd       *****
//*****                All rights reserved                *****
//*************************************************************


#ifndef _CSPEVENT
#define _CSPEVENT

#include "CSPConf.h"
#include "CSPLib.h"
#ifdef _CSP_WINDOWS_
#include <process.h>
#endif



#ifdef _CSP_WINDOWS_
//---------------------- CSPEvent ------------------------------

class CSPLIB_CLASS_EXPORT CSPWaitHandler;

class CSPLIB_CLASS_EXPORT CSPWaitableObject : public CSPObject
{
	friend class CSPLIB_CLASS_EXPORT CSPWaitHandler;

public:

	CSPWaitableObject();
	virtual ~CSPWaitableObject();
	bool Wait( unsigned long ulTime = 0 );

protected:
	HANDLE m_hWaitObj;
	virtual bool OnFireInternal() = 0;
	virtual bool OnTimeoutInternal() = 0;

};


class CSPLIB_CLASS_EXPORT CSPEvent : public CSPWaitableObject
{
	
public:
	bool AutoReset;

	CSPEvent();
	virtual ~CSPEvent();
	virtual void OnFire();
	virtual void OnTimeout();
	bool Fire();
	bool Reset();
	
	bool GetState() const;

protected:
	bool m_bState;

	bool OnFireInternal();
	bool OnTimeoutInternal();
};


//------------------------ CSPEventHandler ---------------------------

class CSPLIB_CLASS_EXPORT CSPWaitHandler : public CSPObject
{
public:
	CSPWaitHandler();
	virtual ~CSPWaitHandler();

	bool Add( CSPWaitableObject& WaitObj );
	void Clear();
	bool Wait();
	unsigned long GetIndex() const;

	unsigned long Timeout;
	bool All;

protected:
	CSPScalarArray<CSPWaitableObject*> m_WaitObjects;
	unsigned long m_ulCount;
	unsigned long m_ulIndex;

};
#endif

#ifdef _CSP_LINUX_
class CSPEvent : public CSPObject
{
	
public:
	bool AutoReset;

	CSPEvent();
	virtual ~CSPEvent();
	virtual void OnFire();
	virtual void OnTimeout();
	bool Fire();
	bool Reset();
	bool GetState() const;
	bool Wait( unsigned long ulTime = 0 );

protected:
	pthread_cond_t m_Cond;
	pthread_mutex_t m_Mutex;
	bool m_bState;
	bool OnFireInternal();
	bool OnTimeoutInternal();
};
#endif

#endif
