//------------------------- CSPMime.h -------------------------

//*************************************************************
//*****       Copyright 2002 - 2008 Micronovae Ltd       *****
//*****                All rights reserved                *****
//*************************************************************


#ifndef _CSPMIME
#define _CSPMIME

#include "CSPConf.h"
#include "CSPLib.h"
#include "CSPString.h"
#include "CSPBuffer.h"
#include "CSPTools.h"
#include "CSPList.h"


class CSPTOOLS_CLASS_EXPORT CSPMimePart : public CSPObject
{
	friend class CSPMime;

public:
	enum
	{
		DISP_ATTACHMENT = 0x1UL,
		DISP_INLINE
	};

	enum
	{
		ENC_BINARY = 0x1UL,
		ENC_BASE64,
		ENC_QUOTED_PRINTABLE,
		ENC_7BIT,
		ENC_8BIT
	};

	CSPMimePart();
	virtual ~CSPMimePart();

	CSPString ContentType;
	unsigned long Disposition;
	CSPString Filename;
	CSPString ContentId;
	unsigned long Encoding;
	CSPBuffer Data;
	const CSPString& GetOutput() const;


	void Clear();
	void Render();
	bool AddHeader( const CSPString& strHeader, const CSPString& strValue );

protected:
	CSPString m_strOutput;
	CSPList<CSPString> m_Headers;

	static void EncodeBase64( const CSPBuffer& Buffer, CSPString& strOutput );
	static void EncodeBase64( const CSPString& strInput, CSPString& strOutput );
	static unsigned char MapBase64( unsigned char val );
	static void EncodeQuotedPrintable( const CSPBuffer& Buffer, CSPString& strOutput );
	static void EncodeQuotedPrintable( const CSPString& strInput, CSPString& strOutput );
	static void Encode7Bit( const CSPBuffer& Buffer, CSPString& strOutput );
	static void Encode7Bit( const CSPString& strInput, CSPString& strOutput );
	static void Encode8Bit( const CSPBuffer& Buffer, CSPString& strOutput );
	static void Encode8Bit( const CSPString& strInput, CSPString& strOutput );
	static char GetHex( unsigned char uchByte );
};

class CSPTOOLS_CLASS_EXPORT CSPMime : public CSPObject
{
	enum
	{
		TYPE_MIXED = 1,
		TYPE_ALTERNATIVE,
		TYPE_CUSTOM = -1
	};

public:
	CSPMime();
	virtual ~CSPMime();
	bool Add( const CSPMimePart& Part );
	CSPMimePart& Get( unsigned long x ) const;
	void Clear();
	unsigned long GetCount() const;
	virtual bool Render();
	const CSPString& GetOutput() const;
	const CSPString& GetBoundary() const;
	virtual CSPString GetContentType() const;
	void SetContentTypeMixed();
	void SetContentTypeAlternative();
	bool SetCustomContentType( const CSPString& strContentType );
	

protected:
	CSPList<CSPMimePart> m_Parts;
	CSPString m_strOutput;
	CSPString m_strBoundary;
	long m_lContentType;
	CSPString m_strCustomContentType;
};

#endif
