//------------------------ CSPSockets.h -----------------------

//*************************************************************
//*****       Copyright 2002 - 2008 Micronovae Ltd       *****
//*****                All rights reserved                *****
//*************************************************************


#ifndef _CSPSOCKETS
#define _CSPSOCKETS

#include "CSPConf.h"
#include "CSPLib.h"
#include "CSPBuffer.h"
#include <winsock2.h>

#ifdef _CSP_WINDOWS_
#define EWOULDBLOCK WSAEWOULDBLOCK 
#define EINPROGRESS WSAEINPROGRESS
#define socklen_t int
#pragma comment(lib,"ws2_32.lib")
#endif

#ifdef _CSP_LINUX_
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>

#define SOCKADDR sockaddr
#define SOCKADDR_IN sockaddr_in
#define SOCKET int
#define ioctlsocket ioctl
#define closesocket close
#define TIMEVAL timeval
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#define SD_BOTH SHUT_RDWR
#endif



class CSPTOOLS_CLASS_EXPORT CSPSocketAddress : public CSPObject
{
public:
	CSPSocketAddress();
	virtual ~CSPSocketAddress();

	const SOCKADDR& GetSocketAddress() const;
	void SetSocketAddress( SOCKADDR& Address );

protected:
	SOCKADDR m_sockaddr;
};


class CSPTOOLS_CLASS_EXPORT CSPIPAddress : public CSPSocketAddress
{
public:
	CSPIPAddress();
	virtual ~CSPIPAddress();

	void SetIPAddress( unsigned char IPB1, unsigned char IPB2, unsigned char IPB3, unsigned char IPB4, unsigned short ushPort );
	void SetIPAddress( unsigned long ulIPAddress, unsigned short ushPort );
	bool SetIPAddressByName( const CSPString& strName, unsigned short ushPort );
	static unsigned long ResolveName( const CSPString& strName );
};

class CSPTOOLS_CLASS_EXPORT CSPSocketBase : public CSPObject
{
public:
	enum
	{
		WAIT_FOREVER = -1
	};

	CSPSocketBase();
	virtual ~CSPSocketBase();

	bool WaitExcept( DWORD dwWaitTime = WAIT_FOREVER ) const;
	bool WaitRead( DWORD dwWaitTime = WAIT_FOREVER ) const;
	bool WaitWrite( DWORD dwWaitTime = WAIT_FOREVER ) const;
	bool Wait( DWORD dwWaitTime = WAIT_FOREVER ) const;
	operator SOCKET() const;
	bool GetBlockingMode() const;
	bool IsValid() const;

	bool Bind( const CSPSocketAddress& Address );
	virtual bool BindAny();
	bool SetBlockingMode( bool bMode );
	bool IsTimeout() const;
	void ClearTimeout();

protected:
	bool m_bBlockingMode;
	SOCKET m_socket;
	bool m_bBound;
	mutable bool m_bTimeout;

	virtual bool SetupSocket() = 0;
	virtual bool CreateSocket() = 0;
	bool SetSocket( SOCKET sSocket, bool bBlockingMode = true );
	int GetError() const;

private:
	CSPSocketBase( const CSPSocketBase& );
	CSPSocketBase& operator = ( const CSPSocketBase& );
};


class CSPTOOLS_CLASS_EXPORT CSPSocket : public CSPSocketBase
{
	friend class CSPTOOLS_CLASS_EXPORT CSPServerSocket;
public:

	CSPSocket();
	virtual ~CSPSocket();

	unsigned long GetBytes( bool bWait = true ) const;
	void Disconnect();
	bool Connect( const CSPSocketAddress& Address );

	virtual bool ReceiveAll( CSPBuffer& Data ) const;
	virtual bool Receive( CSPBuffer& Data, unsigned long ulSize = 0 ) const;
	virtual bool Receive( void* pData, unsigned long ulSize ) const;
	virtual bool Send( const CSPBuffer& Data ) const;
	virtual bool Send( const CSPString& strData ) const;
	virtual bool Send( const void* pData, unsigned long ulSize ) const;

protected:
	mutable DWORD m_dwBytes;
	bool SetupSocket();

private:
	CSPSocket( const CSPSocket& );
	CSPSocket& operator = ( const CSPSocket& );
};


class CSPTOOLS_CLASS_EXPORT CSPTCPSocket : public CSPSocket
{
public:
	CSPTCPSocket();
	virtual ~CSPTCPSocket();

	virtual bool BindAny( unsigned short ushPort );
	virtual bool BindAny();

protected:
	virtual bool CreateSocket();

private:
	CSPTCPSocket( const CSPTCPSocket& );
	CSPTCPSocket& operator = ( const CSPTCPSocket& );
};


class CSPTOOLS_CLASS_EXPORT CSPServerSocket : public CSPSocketBase
{
public:

	CSPServerSocket();
	virtual ~CSPServerSocket();

	bool Accept( CSPSocket& socSocket, CSPSocketAddress& Address ) const;
	bool Accept( CSPSocket& socSocket ) const;
	bool Listen( int iBackLog = SOMAXCONN );

protected:
	bool SetupSocket();
private:
	CSPServerSocket( const CSPServerSocket& );
	CSPServerSocket& operator = ( const CSPServerSocket& );
};


class CSPTOOLS_CLASS_EXPORT CSPTCPServerSocket : public CSPServerSocket
{
public:
	virtual bool BindAny( unsigned short ushPort );
	virtual bool BindAny();
	CSPTCPServerSocket();
	virtual ~CSPTCPServerSocket();

protected:
	virtual bool CreateSocket();

private:
	CSPTCPServerSocket( const CSPServerSocket& );
	CSPTCPServerSocket& operator = ( const CSPServerSocket& );
};


#endif
