//------------------------ CSPThread.h: -----------------------

//*************************************************************
//*****       Copyright 2002 - 2008 Micronovae Ltd       *****
//*****                All rights reserved                *****
//*************************************************************


#ifndef _CSPTHREAD
#define _CSPTHREAD

#include "CSPConf.h"
#include "CSPLib.h"
#include "CSPEvent.h"

#ifdef _CSP_WINDOWS_
#include <process.h>
#endif

#ifdef _CSP_LINUX_
#include <pthread.h>
#endif


//---------------------- CSPThread -------------------------------
#ifdef _CSP_WINDOWS_
class CSPLIB_CLASS_EXPORT CSPThread : public CSPWaitableObject
{
public:

	CSPThread();
	virtual ~CSPThread();

	unsigned int GetID() const;
	unsigned long GetHandle() const;
	bool Start( void* pvUserParam = 0 );
	unsigned long Suspend();
	unsigned long Resume();
	bool SetPriority( int iLevel );
	int GetPriority() const;
	bool Kill( unsigned long ulExitCode = 0 );
	bool Reuse( bool bReuse );

	unsigned long GetExitCode() const;


	enum
	{
		ABOVE_NORMAL = THREAD_PRIORITY_ABOVE_NORMAL,
		BELOW_NORMAL = THREAD_PRIORITY_BELOW_NORMAL,
		HIGHEST = THREAD_PRIORITY_HIGHEST,
		IDLE = THREAD_PRIORITY_IDLE,
		LOWEST = THREAD_PRIORITY_LOWEST,
		NORMAL = THREAD_PRIORITY_NORMAL,
		TIME_CRITICAL = THREAD_PRIORITY_TIME_CRITICAL,
		ERROR_RETURN = THREAD_PRIORITY_ERROR_RETURN,
		WAIT_FOREVER = INFINITE
	};

protected:

	void* GetUserParam() const;
	virtual void Run() = 0;
	virtual void OnExit();
	virtual void OnTimeout();
	virtual void OnFinish();
	virtual void OnStart();
	void SetExitCode( unsigned long ulExitCode );
	void SafeDestruction();
	bool OnFireInternal();
	bool OnTimeoutInternal();
	bool Create();
	bool WaitNoFire();
	

private:

	unsigned int m_uiThreadID;
	unsigned long& m_ulThread;
	CSPEvent m_StartEvent;
	void* m_pvUserParameter;
	mutable int m_iPriorityLevel;
	unsigned long m_ulExitCode;
	bool m_bRunning;
	bool m_bKill;
	bool m_bReuse;
	bool m_bExited;



	CSPThread( const CSPThread& );
	CSPThread& operator = ( const CSPThread& );
	static unsigned int __stdcall ThreadProc( void* pCtx );
};
#endif


#ifdef _CSP_LINUX_
class CSPLIB_CLASS_EXPORT CSPThread : public CSPObject
{
public:

	CSPThread();
	virtual ~CSPThread();

	unsigned int GetID() const;
	bool Start( void* pvUserParam = 0 );
//	unsigned long Suspend();
//	unsigned long Resume();
	bool SetPriority( int iLevel );
	int GetPriority() const;
	bool Kill( unsigned long ulExitCode = 0 );
	bool Reuse( bool bReuse );
	unsigned long GetExitCode() const;


	enum
	{
		ABOVE_NORMAL = 0,
		BELOW_NORMAL = 1,
		HIGHEST = 2,
		IDLE = 3,
		LOWEST = 4,
		NORMAL = 5,
		TIME_CRITICAL = 6,
		ERROR_RETURN = 0XFFFFFFFFUL,
		WAIT_FOREVER = 0XFFFFFFFFUL
	};

protected:

	void* GetUserParam() const;
	virtual void Run() = 0;
	virtual void OnExit();
	virtual void OnTimeout();
	virtual void OnFinish();
	virtual void OnStart();
	void SetExitCode( unsigned long ulExitCode );
	void SafeDestruction();
	bool OnFireInternal();
	bool OnTimeoutInternal();
	bool Create();
	bool WaitNoFire();

private:

	pthread_t m_uiThreadID;
	void* m_pvUserParameter;
	CSPEvent m_StartEvent;
	mutable int m_iPriorityLevel;
	unsigned long m_ulExitCode;
	bool m_bRunning;
	bool m_bKill;
	bool m_bReuse;
	bool m_bExited;


	CSPThread( const CSPThread& );
	CSPThread& operator = ( const CSPThread& );
	static void* ThreadProc( void* pCtx );
};
#endif

#endif
