//------------------------ CSPThreadLock.h: -----------------------

//*************************************************************
//*****       Copyright 2002 - 2008 Micronovae Ltd       *****
//*****                All rights reserved                *****
//*************************************************************


#ifndef _CSPTHREADLOCK
#define _CSPTHREADLOCK

#include "CSPConf.h"
#include "CSPLib.h"

#ifdef _CSP_WINDOWS_
#include <process.h>
#endif

#ifdef _CSP_LINUX_
#include <pthread.h>
#endif




//---------------------- CSPThreadLockBase ---------------------------

class CSPLIB_CLASS_EXPORT CSPThreadLockBase : public CSPObject
{
public:
	CSPThreadLockBase();
	virtual ~CSPThreadLockBase();

	virtual unsigned long Lock() = 0;
	virtual unsigned long Unlock() = 0;
	virtual bool TryLock() = 0;
	virtual unsigned long GetLockCount() const = 0;
	static unsigned long GetCallingThreadID();
	unsigned long GetOwnerThreadID() const;

protected:
	unsigned long m_ulLockCount;
	unsigned long m_OwnerThreadID;

private:
	CSPThreadLockBase( CSPThreadLockBase& );
	CSPThreadLockBase& operator = ( CSPThreadLockBase& );
};


//---------------------- CSPThreadLock ---------------------------

class CSPLIB_CLASS_EXPORT CSPThreadLock : public CSPThreadLockBase
{
public:
	CSPThreadLock();
	virtual ~CSPThreadLock();

	unsigned long Lock();
	unsigned long Unlock();
	bool TryLock();
	unsigned long GetLockCount() const;

protected:
#ifdef _CSP_WINDOWS_
	mutable CRITICAL_SECTION m_CS;
#endif
#ifdef _CSP_LINUX_
	mutable pthread_mutex_t m_Mutex;
#endif

private:
	CSPThreadLock( CSPThreadLock& );
	CSPThreadLock& operator = ( CSPThreadLock& );
};

typedef CSPThreadLock CSPCriticalSection;



//---------------------- CSPThreadLockHandler ---------------------------

class CSPLIB_CLASS_EXPORT CSPThreadLockHandler : public CSPObject
{
public:
	CSPThreadLockHandler();
	CSPThreadLockHandler( CSPThreadLockBase& TL );
	virtual ~CSPThreadLockHandler();

	void Attach( CSPThreadLockBase& TL );
	void Detach();
	unsigned long Lock();
	unsigned long Unlock();
	bool TryLock();
	unsigned long GetLockCount() const;

protected:
	CSPThreadLockBase* pCSPTL;
	unsigned long m_ulCount;
};

#endif
